@@ -2,23 +2,12 @@ package WorePAN;
 
 use strict;
 use warnings;
-use Archive::Any::Lite;
 use File::Temp ();
-use Parse::PMFile;
-use Parse::CPAN::Whois;
-use Path::Extended::Dir;
-use Path::Extended::File;
+use Path::Extended::Tiny ();
 use File::Spec;
-use LWP::UserAgent;
-use JSON;
-use URI;
-use URI::QueryParam;
-use version;
-use CPAN::Meta::YAML;
-use CPAN::DistnameInfo;
-use IO::Zlib;
+use HTTP::Tiny;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 sub new {
   my ($class, %args) = @_;
@@ -29,7 +18,7 @@ sub new {
     $args{root} = File::Temp::tempdir(CLEANUP => 1, ($args{tmp} ? (DIR => $args{tmp}) : TMPDIR => 1));
     warn "'root' is missing; created a temporary WorePAN directory: $args{root}\n" if $args{verbose};
   }
-  $args{root} = Path::Extended::Dir->new($args{root})->mkdir;
+  $args{root} = Path::Extended::Tiny->new($args{root})->mkdir;
   $args{cpan} ||= "http://www.cpan.org/";
   if ($args{use_backpan}) {
     $args{backpan} ||= "http://backpan.cpan.org/";
@@ -38,9 +27,8 @@ sub new {
 
   $args{pid} = $$;
 
-  $args{ua} ||= LWP::UserAgent->new(
+  $args{ua} ||= HTTP::Tiny->new(
     agent => "WorePAN/$VERSION",
-    env_proxy => 1,
   );
 
   my $self = bless \%args, $class;
@@ -55,7 +43,7 @@ sub new {
 
   if (@files) {
     $self->_fetch(\@files);
-    $self->update_indices;
+    $self->update_indices(%args);
   }
 
   $self;
@@ -71,6 +59,42 @@ sub whois { shift->{root}->file('authors/00whois.xml') }
 sub mailrc { shift->{root}->file('authors/01mailrc.txt.gz') }
 sub packages_details { shift->{root}->file('modules/02packages.details.txt.gz') }
 
+sub slurp_whois {
+  my $self = shift;
+  my $index = $self->whois;
+  require Parse::CPAN::Whois;
+  Parse::CPAN::Whois->new($index->path)->authors;
+}
+
+sub slurp_mailrc {
+  my $self = shift;
+  $self->_slurp($self->mailrc);
+}
+sub slurp_packages_details {
+  my $self = shift;
+  $self->_slurp($self->packages_details);
+}
+
+sub _slurp {
+  my ($self, $index) = @_;
+  return unless $index->exists;
+
+  require IO::Zlib;
+  my $fh = IO::Zlib->new($index->path, "rb") or die $!;
+  my @lines;
+  my $done_preambles;
+  while(<$fh>) {
+    chomp;
+    if (/^\s*$/) {
+      $done_preambles = 1;
+      next;
+    }
+    next unless $done_preambles;
+    push @lines, $_;
+  }
+  @lines;
+}
+
 sub add_files {
   my ($self, @files) = @_;
   $self->_fetch(\@files);
@@ -95,7 +119,7 @@ sub _fetch {
   for my $file (@$files) {
     my $dest;
     if (-f $file && $file =~ /\.(?:tar\.(?:gz|bz2)|tgz|zip)$/) {
-      my $source = Path::Extended::File->new($file);
+      my $source = Path::Extended::Tiny->new($file);
       $dest = $_root->file('L/LO/LOCAL/', $source->basename);
       $self->_log("copy $source to $dest");
       $source->copy_to($dest);
@@ -129,6 +153,10 @@ sub _dists2files {
   my ($self, $dists) = @_;
   return unless ref $dists eq ref {};
 
+  require URI;
+  require URI::QueryParam;
+  require JSON::PP;
+
   my $uri = URI->new('http://api.cpanauthors.org/uploads/dist');
   my @keys = keys %$dists;
   my @files;
@@ -139,11 +167,11 @@ sub _dists2files {
     ]);
     $self->_log("called API: $uri");
     my $res = $self->{ua}->get($uri);
-    if ($res->is_error) {
-      warn "API error: $uri ".$res->status_line;
+    if (!$res->{success}) {
+      warn "API error: $uri $res->{status} $res->{reason}";
       return;
     }
-    my $rows = eval { JSON::decode_json($res->decoded_content) };
+    my $rows = eval { JSON::PP::decode_json($res->{content}) };
     if ($@) {
       warn $@;
       return;
@@ -176,7 +204,7 @@ sub __fetch {
   $dest->parent->mkdir;
 
   if ($self->{local_mirror}) {
-    my $source = Path::Extended::File->new($self->{local_mirror}, "authors/id", $file);
+    my $source = Path::Extended::Tiny->new($self->{local_mirror}, "authors/id", $file);
     if ($source->exists) {
       $self->_log("copy $source to $dest");
       $source->copy_to($dest);
@@ -188,12 +216,12 @@ sub __fetch {
     my $url = $self->{cpan}."authors/id/$file";
     $self->_log("mirror $url to $dest");
     my $res = $self->{ua}->mirror($url => $dest);
-    return $dest if $res->is_success;
+    return $dest if $res->{success};
     if ($self->{backpan}) {
       my $url = $self->{backpan}."authors/id/$file";
       $self->_log("mirror $url to $dest");
       my $res = $self->{ua}->mirror($url => $dest);
-      return $dest if $res->is_success;
+      return $dest if $res->{success};
     }
   }
   warn "Can't fetch $file\n";
@@ -204,6 +232,9 @@ sub walk {
   my ($self, %args) = @_;
   my $root = $self->{root}->subdir('authors/id');
   my $tmproot = $self->{tmp} || $args{tmp};
+  my $allow_dev_releases = $args{developer_releases} || $self->{developer_releases};
+
+  require Archive::Any::Lite;
 
   local $Archive::Any::Lite::IGNORE_SYMLINK = 1;
   $root->recurse(callback => sub {
@@ -214,13 +245,13 @@ sub walk {
     my $basename = $archive_file->basename;
     return unless $basename =~ /\.(?:tar\.(?:gz|bz2)|tgz|zip)$/;
     return if $basename =~ /^perl\-\d+/; # perls
-    return if !$args{developer_releases} && (
+    return if !$allow_dev_releases && (
          $basename =~ /\d\.\d+_\d/  # dev release
       or $basename =~ /TRIAL/       # trial release
     );
 
     my $archive = Archive::Any::Lite->new($archive_file->path);
-    my $tmpdir = Path::Extended::Dir->new(File::Temp::tempdir(CLEANUP => 1, ($tmproot ? (DIR => $tmproot) : (TMPDIR => 1))));
+    my $tmpdir = Path::Extended::Tiny->new(File::Temp::tempdir(CLEANUP => 1, ($tmproot ? (DIR => $tmproot) : (TMPDIR => 1))));
     $archive->extract($tmpdir);
     my $basedir = $tmpdir->children == 1 ? ($tmpdir->children)[0] : $tmpdir;
     $basedir = $tmpdir unless -d $basedir;
@@ -232,12 +263,19 @@ sub walk {
 }
 
 sub update_indices {
-  my $self = shift;
+  my ($self, %args) = @_;
 
   return if $self->{no_indices};
 
+  require IO::Zlib;
+  require Parse::PMFile;
+  require Parse::LocalDistribution;
+
+  my $allow_dev_releases = $args{developer_releases} || $self->{developer_releases};
+  my $permissions = $args{permissions} || $self->{permissions};
+
   my (%authors, %packages);
-  $self->walk(callback => sub {
+  $self->walk(%args, callback => sub {
     my ($basedir, $path, $archive_file) = @_;
 
     my $mtime = $archive_file->mtime;
@@ -248,63 +286,14 @@ sub update_indices {
     # see PAUSE::dist::mail_summary
     return if $basedir->basename eq 'blib' or $basedir->subdir('blib')->exists;
 
-    my ($metafile, @pmfiles);
-    $basedir->recurse(callback => sub {
-      my $file = shift;
-      push @pmfiles, $file if $file =~ /\.pm(?:\.PL)?$/i;
-      $metafile ||= $file if $file =~ /META.(?:yml|json)$/;
-    });
-
-    my $meta;
-    if ($metafile) {
-      my $content = do { local $/; open my $fh, '<:utf8', $metafile; <$fh> };
-      if ($metafile =~ /\.yml$/) {
-        $meta = eval { CPAN::Meta::YAML->read_string($content)->[0] };
-      } else {
-        $meta = eval { JSON::decode_json($content) };
-      }
-    }
-
-    # Provides field has precedence; should also check meta_ok
-    if ($self->_version_from_meta_ok($meta)) {
-      $self->_update_packages(\%packages, $meta->{provides}, $path, $mtime);
-      return;
-    }
-
-    my $parser = Parse::PMFile->new($meta);
-PMFILES:
-    for my $pmfile (@pmfiles) {
-      my $relpath = $pmfile->relative($basedir);
-
-      # adopted from PAUSE::dist::filter_pms
-      next if $relpath =~ m!^(?:x?t|inc|local|perl5)!;
-
-      if ($meta) {
-        my $no_index = $meta->{no_index} || $meta->{private};
-        if (ref $no_index eq 'HASH') {
-          my %map = (
-            file => qr{\z},
-            directory => qr{/},
-          );
-          for my $k (qw(file directory)) {
-            next unless my $v = $no_index->{$k};
-            my $rest = $map{$k};
-            if (ref $v eq 'ARRAY') {
-              for my $ve (@$v) {
-                $ve =~ s|/+$||;
-                next PMFILES if $relpath =~ /^$ve$rest/;
-              }
-            } else {
-              $v =~ s|/+$||;
-              next PMFILES if $relpath =~ /^$v$rest/;
-            }
-          }
-        }
-      }
-
-      my $info = $parser->parse($pmfile);
-      $self->_update_packages(\%packages, $info, $path, $mtime);
+    my $args = {ALLOW_DEV_VERSION => $allow_dev_releases};
+    if ($permissions) {
+      $args->{PERMISSIONS} = $permissions;
+      $args->{USERID} = $author;
     }
+    my $parser = Parse::LocalDistribution->new($args);
+    my $info = $parser->parse($basedir);
+    $self->_update_packages(\%packages, $info, $path, $mtime);
   });
   $self->_write_whois(\%authors);
   $self->_write_mailrc(\%authors);
@@ -313,31 +302,17 @@ PMFILES:
   return 1;
 }
 
-# borrowed from PAUSE::dist
-sub _version_from_meta_ok {
-  my ($self, $meta) = @_;
-  return unless $meta && ref $meta eq ref {};
-
-  my $provides = $meta->{provides};
-  return unless $provides && ref $provides eq ref {} && %$provides;
-
-  my ($mb_v) = ($meta->{generated_by} || '') =~ /Module::Build version ([\d\.]+)/;
-  return 1 unless $mb_v;
-  return 1 if $mb_v eq '0.250.0';
-  return if $mb_v >= 0.19 && $mb_v < 0.26;
-  return 1;
-}
-
 sub _update_packages {
   my ($self, $packages, $info, $path, $mtime) = @_;
 
   for my $module (sort keys %$info) {
+    next unless exists $info->{$module}{version};
+    my $new_version = $info->{$module}{version};
     if (!$packages->{$module}) { # shortcut
-      $packages->{$module} = [$info->{$module}{version}, $path, $mtime];
+      $packages->{$module} = [$new_version, $path, $mtime];
       next;
     }
     my $ok = 0;
-    my $new_version = $info->{$module}{version};
     my $cur_version = $packages->{$module}[0];
     if (Parse::PMFile->_vgt($new_version, $cur_version)) {
       $ok++;
@@ -420,18 +395,7 @@ sub look_for {
 
   return unless defined $package;
 
-  my $index = $self->packages_details;
-  return [] unless $index->exists;
-  my $fh = IO::Zlib->new($index->path, "rb") or die $!;
-
-  my $done_preambles = 0;
-  while(<$fh>) {
-    chomp;
-    if (/^\s*$/) {
-      $done_preambles = 1;
-      next;
-    }
-    next unless $done_preambles;
+  for ($self->slurp_packages_details) {
     if (/^$package\s+(\S+)\s+(\S+)$/) {
       return wantarray ? ($1, $2) : $1;
     }
@@ -444,13 +408,9 @@ sub authors { shift->_authors_whois }
 sub _authors_mailrc {
   my $self = shift;
 
-  my $index = $self->mailrc;
-  return [] unless $index->exists;
-  my $fh = IO::Zlib->new($index->path, "rb") or die $!;
-
   my @authors;
-  while(defined(my $line = <$fh>)) {
-    my ($id, $name, $email) = $line =~ /^alias\s+(\S+)\s+"?(.+?)\s+(\S+?)"?\s*$/;
+  for ($self->slurp_mailrc) {
+    my ($id, $name, $email) = /^alias\s+(\S+)\s+"?(.+?)\s+(\S+?)"?\s*$/;
     next unless $id;
     $email =~ tr/<>//d;
     push @authors, {pauseid => $id, name => $name, email => $email};
@@ -461,10 +421,8 @@ sub _authors_mailrc {
 sub _authors_whois {
   my $self = shift;
 
-  my $index = $self->whois;
-  return [] unless $index->exists;
   my @authors;
-  for (Parse::CPAN::Whois->new($index->path)->authors) {
+  for ($self->slurp_whois) {
     push @authors, {
       pauseid => $_->pauseid,
       name => $_->name,
@@ -479,20 +437,8 @@ sub _authors_whois {
 sub modules {
   my $self = shift;
 
-  my $index = $self->packages_details;
-  return [] unless $index->exists;
-  my $fh = IO::Zlib->new($index->path, "rb") or die $!;
-
   my @modules;
-  my $done_preambles = 0;
-  while(<$fh>) {
-    chomp;
-    if (/^\s*$/) {
-      $done_preambles = 1;
-      next;
-    }
-    next unless $done_preambles;
-
+  for ($self->slurp_packages_details) {
     /^(\S+)\s+(\S+)\s+(\S+)/ or next;
     push @modules, {module => $1 ,version => $2 eq 'undef' ? undef : $2, file => $3};
   }
@@ -501,20 +447,9 @@ sub modules {
 
 sub files {
   my $self = shift;
-  my $index = $self->packages_details;
-  return [] unless $index->exists;
-  my $fh = IO::Zlib->new($index->path, "rb") or die $!;
 
   my %files;
-  my $done_preambles = 0;
-  while(<$fh>) {
-    chomp;
-    if (/^\s*$/) {
-      $done_preambles = 1;
-      next;
-    }
-    next unless $done_preambles;
-
+  for ($self->slurp_packages_details) {
     /^\S+\s+\S+\s+(\S+)/ or next;
     $files{$1} = 1;
   }
@@ -524,6 +459,8 @@ sub files {
 sub latest_distributions {
   my $self = shift;
 
+  require CPAN::DistnameInfo;
+
   my %dists;
   for my $file (@{ $self->files || [] }) {
     my $dist = CPAN::DistnameInfo->new($file);
@@ -633,6 +570,14 @@ If set to true, WorePAN removes its contents when the instance is gone (mainly f
 
 If set to true, WorePAN won't create/update indices.
 
+=item developer_releases
+
+WorePAN usually ignores developer releases and doesn't fetch/index them. Set this to true if you need them for whatever reasons.
+
+=item permissions
+
+If you pass a valid PAUSE::Permissions instance, it will be used while indexing.
+
 =item tmp
 
 If set, temporary directories will be created in the specified directory.
@@ -674,27 +619,27 @@ Creates/updates mailrc and packages_details indices.
     });
   });
 
-Walks down the WorePAN directory and extracts each distribution into a temporary directory, and runs a callback to which a Path::Extended::Dir object for the directory is passed as an argument. Used internally to create indices.
+Walks down the WorePAN directory and extracts each distribution into a temporary directory, and runs a callback to which a Path::Extended::Tiny object for the directory is passed as an argument. Used internally to create indices.
 
 =head2 root
 
-returns a L<Path::Extended::Dir> object that represents the root path you specified (or created internally).
+returns a L<Path::Extended::Tiny> object that represents the root path you specified (or created internally).
 
 =head2 file
 
-takes a relative path to a distribution ("P/PA/PAUSE/distribution.tar.gz") and returns a L<Path::Extended::File> object.
+takes a relative path to a distribution ("P/PA/PAUSE/distribution.tar.gz") and returns a L<Path::Extended::Tiny> object.
 
-=head2 whois
+=head2 whois, slurp_whois
 
-returns a L<Path::Extended::File> object that represents the "00whois.xml" file.
+returns a L<Path::Extended::Tiny> object that represents the "00whois.xml" file. C<slurp_whois> returns a list of author entries parsed by L<Parse::CPAN::Whois>.
 
-=head2 mailrc
+=head2 mailrc, slurp_mailrc
 
-returns a L<Path::Extended::File> object that represents the "01mailrc.txt.gz" file.
+returns a L<Path::Extended::Tiny> object that represents the "01mailrc.txt.gz" file. C<slurp_mailrc> returns a list of lines without preambles.
 
-=head2 packages_details
+=head2 packages_details, slurp_packages_details
 
-returns a L<Path::Extended::File> object that represents the "02packages.details.txt.gz" file.
+returns a L<Path::Extended::Tiny> object that represents the "02packages.details.txt.gz" file. C<slurp_packages_details> returns a list of lines without preambles.
 
 =head2 look_for
 
